/*
 * Created on 10.09.2004
 *
 */
package de.lunqual.rzpro.ausgabe;

import java.util.ArrayList;
import java.util.Date;

import javax.swing.table.AbstractTableModel;

import de.lunqual.rzpro.database.DBBuchung;
import de.lunqual.rzpro.fenster.buchungsfenster.StatistikFenster;
import de.lunqual.rzpro.items.buchung.BuchungsItem;
import de.lunqual.rzpro.items.buchung.BuchungsListe;
import de.lunqual.rzpro.items.buchung.SaldenItem;
import de.lunqual.rzpro.items.stichworte.StichwortListe;
import de.lunqual.rzpro.RzPro;

/**
 * @author labor
 *
 */
public class StatistikDrucken extends ReportAusgabe {

    BuchungsListe			bl;
    SaldenItem				saldo;
    int rezeptur;
    int mode ;
    Date von;
    Date bis;
    String benutzer;
    String losnummer;
    int	auswahl;

    public static final String DEFAULT_STATISTIK_ORIENTATION = "landscape";

    public StatistikDrucken(RzPro r,Object reportData ,SaldenItem s,String optionKey,java.awt.Window w,
            int auswahl,int rezeptur,int mode ,Date von, Date bis,String benutzer,String losnummer) {
        super(r,reportData,optionKey,w);
        this.rezeptur = rezeptur;
        this.mode=mode;
        this.von=von;
        this.bis=bis;
        this.benutzer=benutzer;
        this.losnummer=losnummer;
        this.auswahl=auswahl;

        rz = r;
        bl = (BuchungsListe)reportData;
        setSaldo(s);
        this.optionKey=optionKey;
    }



    /** liefert ein TableModel
     *
     */
    @Override
	public void  createModel(){
        BuchungsItem bi;
        String codeString="";
        final tableModel modell = new tableModel();
        if (modell != null) {
            for(int i = 0;i < bl.size();i++) {
                bi = bl.getItem(i);
                switch(bi.getCode()) {
                	case DBBuchung.CODE_EINKAUF : codeString=rz.getLocale().getString("report.statistik.code_einkauf");break;
                	case DBBuchung.CODE_ENTNAHME : codeString=rz.getLocale().getString("report.statistik.code_entnahme");break;
                	case DBBuchung.CODE_HERSTELLUNG : codeString=rz.getLocale().getString("report.statistik.code_herstellung");break;
                	case DBBuchung.CODE_KORREKTUR_ZUGANG : codeString=rz.getLocale().getString("report.statistik.code_korrektur");break;
                	case DBBuchung.CODE_KORREKTUR_ABGANG : codeString=rz.getLocale().getString("report.statistik.code_korrektur");break;
                	case DBBuchung.CODE_SCHWUND : codeString=rz.getLocale().getString("report.statistik.code_schwund");break;
                	case DBBuchung.CODE_SCHWUNDAUSGLEICH : codeString=rz.getLocale().getString("report.statistik.code_schwundausgleich");break;
                }
            	String r = ""; 
        		if(bi.getHerstellung() != 0) {
            		r=bi.getHerstellung_losnummer();
            		if(!bi.getHerstellung_bezeichnung().equals("")) {
            			r +=" [" + bi.getHerstellung_bezeichnung()+"]";
            		}
            		if(!bi.getHerstellungProbe().equals("")) {
            			r +=" [" + rz.getLocale().getString("statistik.probe") + ":" + bi.getHerstellungProbe()+"]";
            		}
        		}
        		StichwortListe adressen = rz.getDatabase().getRezeptur().getAdressen(bi.getRezeptur_id());
                modell.addItem(
                        rz.getDatabase().dbGetUser(),
                        new Date(),
                        new Integer(bi.getTyp()),
                        ((auswahl == StatistikFenster.AUSWAHL_ALLE) || (auswahl == StatistikFenster.AUSWAHL_ZUGANG)) ? rz.getLocale().getString("string_ja") : rz.getLocale().getString("string_nein"),
                        ((auswahl == StatistikFenster.AUSWAHL_ALLE) || (auswahl == StatistikFenster.AUSWAHL_ABGANG)) ? rz.getLocale().getString("string_ja") : rz.getLocale().getString("string_nein"),
                        ((benutzer != null) && !benutzer.equals("")) ? benutzer : "",
                        ((losnummer != null) && ! losnummer.equals("")) ? losnummer : "",
                        von,
                        bis,
                        bi.getDatum(),
                        bi.getTyp() == DBBuchung.TYP_ZUGANG ? rz.getLocale().getString("report.statistik.zugang") : rz.getLocale().getString("report.statistik.abgang"),
                        codeString,
                        //bi.getHerstellung_losnummer(),
                        r,
                        bi.getUser(),
                        bi.getBezeichnung(),
                        new Double(bi.getStaerke()),
                        new Double(bi.getLitergewicht()),
                        bi.getLosnummer(),
                        new Double(bi.getLiter()),
                        new Double(bi.getKg()),
                        new Double(bi.getLa()),
                        new Double(bi.getRest_liter()),
                        new Double(bi.getRest_kg()),
                        new Double(bi.getRest_la()),
                        bi.getComment(),
                        new Double(saldo.getLiter()),
                        new Double(saldo.getKg()),
                        new Double(saldo.getLa()),
                        adressen != null?adressen.toVerboseString():""
                		);
            }
	        report = getReport();
	        report.setDataFactory(new org.jfree.report.TableDataFactory ("default", modell));
	        report.setQuery ("default");
        }
    }

    public void setSaldo(SaldenItem s) {
        saldo = new SaldenItem(s.getLiter(),s.getKg(),s.getLa(),s.getBestandseinheit());
    }

    private class tableModel extends AbstractTableModel{

        private final String[] COLUMN_NAMES = {
               "user","datum","abgang",
               "auswahl_zugang","auswahl_abgang","auswahl_benutzer","auswahl_losnummer",
               "auswahl_von","auswahl_bis",
               "buchungsdatum","typ","code","herstellung_losnummer",
               "benutzer","bezeichnung","staerke","litergewicht","losnummer",
               "liter","kg","la","rest_liter","rest_kg","rest_la",
               "bemerkungen",
               "saldo_liter","saldo_kg","saldo_la","adressen"
              };

              private final Class[] COLUMN_TYPE = {
                String.class,java.util.Date.class,Integer.class,
                String.class,String.class,String.class,String.class,
                java.util.Date.class,java.util.Date.class,
                java.util.Date.class,String.class,String.class,String.class,
                String.class,String.class,Double.class,Double.class,String.class,
                Double.class,Double.class,Double.class,Double.class,Double.class,Double.class,
                String.class,
                Double.class,Double.class,Double.class,String.class
              };

              private final ArrayList liste;

        tableModel() {
            liste = new ArrayList();
        }

        public void addItem (
                final String user,
                final java.util.Date datum,
                final Integer abgang,
                final String auswahl_zugang,
                final String auswahl_abgang,
                final String auswahl_benutzer,
                final String auswahl_losnummer,
                final Date auswahl_von,
                final Date auswahl_bis,
                final Date buchungsdatum,
                final String typ,
                final String code,
                final String herstellung_losnummer,
                final String benutzer,
                final String bezeichnung,
                final Double staerke,
                final Double litergewicht,
                final String losnummer,
                final Double liter,
                final Double kg,
                final Double la,
                final Double rest_liter,
                final Double rest_kg,
                final Double rest_la,
                final String bemerkungen,
                final Double saldo_liter,
                final Double saldo_kg,
                final Double saldo_la,
                final String adressen
                )
        {
            	liste.add (new Object[] {
            	        user,datum,abgang,
                          auswahl_zugang,
                          auswahl_abgang,
                          auswahl_benutzer,
                          auswahl_losnummer,
                          auswahl_von,
                          auswahl_bis,
                          buchungsdatum,
                          typ,
                          code,
                          herstellung_losnummer,
                          benutzer,
                          bezeichnung,
                          staerke,
                          litergewicht,
                          losnummer,
                          liter,
                          kg,
                          la,
                          rest_liter,
                          rest_kg,
                          rest_la,
                          bemerkungen,
                          saldo_liter,
                          saldo_kg,
                          saldo_la,
                          adressen
            	});
        }

        @Override
		public Class getColumnClass (final int columnIndex)
        {
          return COLUMN_TYPE[columnIndex];
        }

        public int getColumnCount() {
            return COLUMN_TYPE.length;
        }

        @Override
		public String getColumnName (final int columnIndex)
        {
          return COLUMN_NAMES[columnIndex];
        }

        public int getRowCount ()
        {
          return liste.size();
        }

        public Object getValueAt (final int rowIndex, final int columnIndex)
        {
          final Object[] l = (Object[])liste.get (rowIndex);
          return l[columnIndex];
        }
    }
}


