/*
 *  Copyright (C) 2004 Karlheinz Klingbeil (lunqual)
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package de.lunqual.rzpro.designs;

import java.util.ArrayList;

import javax.swing.JComboBox;

import com.jgoodies.looks.plastic.*;
import com.jgoodies.looks.plastic.theme.BrownSugar;
import com.jgoodies.looks.plastic.theme.DarkStar;
import com.jgoodies.looks.plastic.theme.DesertBlue;
import com.jgoodies.looks.plastic.theme.DesertBluer;
import com.jgoodies.looks.plastic.theme.DesertGreen;
import com.jgoodies.looks.plastic.theme.DesertRed;
import com.jgoodies.looks.plastic.theme.DesertYellow;
import com.jgoodies.looks.plastic.theme.Silver;
import com.jgoodies.looks.plastic.theme.SkyBlue;
import com.jgoodies.looks.plastic.theme.SkyBluer;
import com.jgoodies.looks.plastic.theme.SkyGreen;
import com.jgoodies.looks.plastic.theme.SkyKrupp;
import com.jgoodies.looks.plastic.theme.SkyPink;
import com.jgoodies.looks.plastic.theme.SkyRed;
import com.jgoodies.looks.plastic.theme.SkyYellow;


//java-pakete
import javax.swing.UIManager;
import de.lunqual.rzpro.*;
import de.lunqual.rzpro.items.design.DesignItem;
import de.lunqual.rzpro.log.LogFactory;

/**
 * @author lunqual
 * kennt eine Reihe von Look- and Feels zum Einstellen
 */
public class DesignFactory {
	
	private static final String DESIGN_PLASTIC = "Plastic";
	private static final String DESIGN_PLASTIC_3D = "Plastic3D";
	private static final String DESIGN_PLASTIC_XP = "PlasticXP";
		
	private static final String THEME_SKYBLUE = "SkyBlue";
	private static final String THEME_SKYBLUEER = "SkyBluer";
	private static final String THEME_SKYRED = "SkyRed";
	private static final String THEME_SKYKRUPP = "SkyKrupp";
	private static final String THEME_SKYPINK = "SkyPink";
	private static final String THEME_SKYYELLOW = "SkyYellow";	
	private static final String THEME_SILVER = "Silver";
	private static final String THEME_DARKSTAR = "DarkStar";
	private static final String THEME_SKYGREEN = "SkyGreen";
	private static final String THEME_BROWNSUGAR = "BrownSugar";
	private static final String THEME_DESERTBLUE = "DesertBlue";
	private static final String THEME_DESERTBLUER = "DesertBluer";
	private static final String THEME_DESERTRED = "DesertRed";
	private static final String THEME_DESERTGREEN = "DesertGreen";
	private static final String THEME_DESERTYELLOW = "DesertYellow";
	
	RzPro				rz;
	ArrayList			designListe;
	ArrayList			themeListe;
	
	public DesignFactory(RzPro r) {
		rz = r;
		designListe = new ArrayList();
		themeListe = new ArrayList();
		int i;
		
		UIManager.LookAndFeelInfo[] info = UIManager.getInstalledLookAndFeels();
		for(i = 0;i < info.length;i++) {
			designListe.add(new DesignItem(true,info[i].getName(),info[i].getClassName()));
		}
		designListe.add(new DesignItem(false,DESIGN_PLASTIC,PlasticLookAndFeel.class.getName()));
		designListe.add(new DesignItem(false,DESIGN_PLASTIC_3D,Plastic3DLookAndFeel.class.getName()));
		designListe.add(new DesignItem(false,DESIGN_PLASTIC_XP,PlasticXPLookAndFeel.class.getName()));

		themeListe.add(new DesignItem(false,THEME_SILVER,Silver.class.getName()));
		themeListe.add(new DesignItem(false,THEME_SKYBLUE,SkyBlue.class.getName()));
		themeListe.add(new DesignItem(false,THEME_SKYBLUEER, SkyBluer.class.getName()));
		themeListe.add(new DesignItem(false,THEME_SKYRED,SkyRed.class.getName()));
		themeListe.add(new DesignItem(false,THEME_SKYGREEN,SkyGreen.class.getName()));
		themeListe.add(new DesignItem(false,THEME_SKYYELLOW,SkyYellow.class.getName()));
		themeListe.add(new DesignItem(false,THEME_SKYKRUPP,SkyKrupp.class.getName()));
		themeListe.add(new DesignItem(false,THEME_SKYPINK,SkyPink.class.getName()));
		themeListe.add(new DesignItem(false,THEME_DARKSTAR,DarkStar.class.getName()));
		themeListe.add(new DesignItem(false,THEME_BROWNSUGAR,BrownSugar.class.getName()));
		themeListe.add(new DesignItem(false,THEME_DESERTBLUE,DesertBlue.class.getName()));
		themeListe.add(new DesignItem(false,THEME_DESERTBLUER,DesertBluer.class.getName()));
		themeListe.add(new DesignItem(false,THEME_DESERTRED,DesertRed.class.getName()));
		themeListe.add(new DesignItem(false,THEME_DESERTGREEN,DesertGreen.class.getName()));
		themeListe.add(new DesignItem(false,THEME_DESERTYELLOW,DesertYellow.class.getName()));
	}
	
	public void setDesign(String design,String theme) {
		DesignItem d=null;
		DesignItem t=null;
		if(design == null || design.equals("")) {
			d = (DesignItem)designListe.get(0);
		} else {
			for(int i =0;i<designListe.size();i++) {
				if(((DesignItem)designListe.get(i)).getName().equals(design)){
					d = (DesignItem)designListe.get(i);
					break;
				}
			}	
		}
		if(theme == null || theme.equals("")) {
			d = (DesignItem)themeListe.get(0);
		} else {
			for(int i =0;i<themeListe.size();i++) {
				if(((DesignItem)themeListe.get(i)).getName().equals(theme)){
					t = (DesignItem)themeListe.get(i);
					break;
				}
			}
		}
		setDesign(d,t);
	}
	/**
	 * Das Design setzen
	 * @param design
	 */
	public void setDesign(DesignItem design,DesignItem theme) {
		try {
			if(design != null) {
				if (design.isSystem()) {
					UIManager.setLookAndFeel(design.getClassName());
				} else {
					if(theme != null && !theme.getName().equals("")) {
						PlasticTheme th = getPlasticTheme(theme);
						if(th != null) {
							try {
							PlasticLookAndFeel.setCurrentTheme(th);
							}catch ( Exception e) {}
						}
					}
					if(design.getName().equals(DESIGN_PLASTIC)) {
						UIManager.setLookAndFeel(new PlasticLookAndFeel());					
					} else if(design.getName().equals(DESIGN_PLASTIC_3D)) {
						UIManager.setLookAndFeel(new Plastic3DLookAndFeel());							
					} else if(design.getName().equals(DESIGN_PLASTIC_XP)) {
						UIManager.setLookAndFeel(new PlasticXPLookAndFeel());		
					}
				}
				
				if(rz.getFensterFactory() != null) {
					// es sind schon Fenster geöffnet....
					rz.getFensterFactory().getFensterListe().changeDesign();
				}
			}
		} catch (Exception e) {
			rz.getLogFactory().logMessage(LogFactory.LOG_WARN,"DesignControl.setDesign","could not set design:"+String.valueOf(design)+ "/"+e.getLocalizedMessage());
		}
	}
	
	/**
	 * 
	 */
	private PlasticTheme getPlasticTheme(DesignItem theme) {
		PlasticTheme th = null;
		if(theme != null) {
			if(!theme.isSystem()) {
				if(theme.getName().equals(THEME_SKYBLUE)) {
					th = new SkyBlue();
				} else 	if(theme.getName().equals(THEME_SKYBLUE)) {
					th = new SkyBlue();
				} else 	if(theme.getName().equals(THEME_SILVER)) {
					th = new Silver();
				}else if(theme.getName().equals(THEME_SKYRED)) {
					th = new SkyRed();
				}else if(theme.getName().equals(THEME_SKYKRUPP)) {
					th = new SkyKrupp();
				}else if(theme.getName().equals(THEME_SKYGREEN)) {
					th = new SkyGreen();
				}else if(theme.getName().equals(THEME_SKYYELLOW)) {
					th = new SkyYellow();
				}else if(theme.getName().equals(THEME_SKYPINK)) {
					th = new SkyPink();
				}else if(theme.getName().equals(THEME_DARKSTAR)) {
					th = new DarkStar();
				}else if(theme.getName().equals(THEME_BROWNSUGAR)) {
					th = new BrownSugar();
				}else if(theme.getName().equals(THEME_SKYBLUEER)) {
					th = new SkyBluer();
				}else if(theme.getName().equals(THEME_DESERTBLUE)) {
					th = new DesertBlue();
				}else if(theme.getName().equals(THEME_DESERTBLUER)) {
					th = new DesertBluer();
				}else if(theme.getName().equals(THEME_DESERTRED)) {
					th = new DesertRed();
				}else if(theme.getName().equals(THEME_DESERTGREEN)) {
					th = new DesertGreen();
				}else if(theme.getName().equals(THEME_DESERTYELLOW)) {
					th = new DesertYellow();
				}
			}
		}
		return th;
	}
	/**
	 * eine Combobox mit den vorhandenen Designs füllen
	 * @param cmb
	 * @param def  Default-Design
	 */
	public void fillDesignCmb(JComboBox cmb, String def) {
		int i;
		int select = -1;
		cmb.removeAllItems();
		for(i=0;i < designListe.size();i++) {
			cmb.addItem(designListe.get(i));
			if(((DesignItem)designListe.get(i)).getName().equalsIgnoreCase(def)) {
				select=i;
			}
		}
		if(select != -1) {
			cmb.setSelectedIndex(select);
		}
	}
	
	/**
	 * eine Combobox mit den vorhandenen themes füllen
	 * @param cmb
	 * @param def  Default-Design
	 */
	public void fillThemeCmb(JComboBox cmb, String def) {
		int i;
		int select = -1;
		cmb.removeAllItems();
		for(i=0;i < themeListe.size();i++) {
			cmb.addItem(themeListe.get(i));
			if(((DesignItem)themeListe.get(i)).getName().equalsIgnoreCase(def)) {
				select=i;
			}
		}
		if(select != -1) {
			cmb.setSelectedIndex(select);
		}
	}
	
}
